Package: khaos
Type: Package
Title: Bayesian Sparse Polynomial Chaos Expansion
Version: 2.1.1
Authors@R: c(
    person(given = "Kellin",
           family = "Rumsey",
           role = c("aut", "cre"),
           email = "knrumsey@lanl.gov",
           comment = c(ORCID = "0000-0002-2989-965X")),
    person(given = "J. Derek", 
           family = "Tucker", 
           role = "ctb", 
           email = "jdtuck@sandia.gov", 
           comment = c(ORCID = "0000-0001-8844-2169")))
Description: Implements Bayesian polynomial chaos expansion methods for
    surrogate modeling, uncertainty quantification, and sensitivity
    analysis. Includes sparse regression-based approaches and adaptive
    Bayesian models based on reversible-jump Markov chain Monte Carlo.
    Optional screening and basis-enrichment strategies are provided to
    improve scalability in moderate to high dimensions.
Imports: glmnet, mvtnorm
Suggests: knitr, rmarkdown, lhs, testthat (>= 2.1.0)
License: BSD_3_clause + file LICENSE
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.2
Packaged: 2026-04-22 15:22:43 UTC; knrumsey
Author: Kellin Rumsey [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2989-965X>),
  J. Derek Tucker [ctb] (ORCID: <https://orcid.org/0000-0001-8844-2169>)
Maintainer: Kellin Rumsey <knrumsey@lanl.gov>
Repository: CRAN
Date/Publication: 2026-04-23 20:30:03 UTC
Built: R 4.6.0; ; 2026-04-26 08:20:24 UTC; unix
