# Generated by roxygen2: do not edit by hand

S3method(plot,adaptive_khaos)
S3method(plot,ordinal_khaos)
S3method(plot,sparse_khaos)
S3method(predict,adaptive_khaos)
S3method(predict,ordinal_khaos)
S3method(predict,sparse_khaos)
S3method(print,sparse_khaos)
export(adaptive_khaos)
export(adaptive_khaos2)
export(adaptive_khaos_gprior)
export(adaptive_khaos_ridge)
export(ordinal_khaos)
export(sobol_khaos)
export(sparse_khaos)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(stats,dgamma)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,rbeta)
importFrom(stats,rgamma)
importFrom(stats,runif)
importFrom(stats,var)
